﻿Option Explicit On
Option Strict On

Imports System.Net.Sockets
Imports System.Text
Imports System.Threading
Imports Microsoft.VisualBasic.FileIO
Imports System.Net
Imports System.IO

''' <summary>
''' frmMain represents the program's main form.
''' </summary>
''' <remarks>Copyright (c) 2015   Vanderhaegen Bart</remarks>
Public Class frmMain

    ' Define global instance of myServer
    Private WithEvents myServer As Server = Server.getServer()
    Private serverInterpreter As ServerFunctions

    ' GUI control arrays
    Private mOutputCheckboxes As CheckBox()
    Private mInputTextboxes As TextBox()

    ' Define the K8055 card Object
    Private mK8055 As New K8055()
    Private mIsLoaded As Boolean = False

    ' Class constants
    Private Const MAX_LISTITEMS As Integer = 2000       ' Max items in the listbox
    Private Const REMOTE_IP_URL As String = "http://www.bvsystems.be/tools/ip.php"
    Private Const START_SERVER As String = "Start Server"
    Private Const STOP_SERVER As String = "Stop Server"



#Region "Form Events"
    ''' <summary>
    ''' Form load event.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub Form1_Load() Handles MyBase.Load

        mOutputCheckboxes = New CheckBox() {OUT1, OUT2, OUT3, OUT4, OUT5, OUT6, OUT7, OUT8}
        mInputTextboxes = New TextBox() {inpPort, inpPassword, inpUser}

        ' Set our interpreter class
        serverInterpreter = New ServerFunctions(mK8055)
        myServer.interpreter = serverInterpreter
        myServer.dlgCommand = AddressOf callback_instruction

        loadAddresses()
        prepareLists()
        readFromRegistry()

        lblIP.Text = String.Format("Server IP: {0}",
                                    GetIPv4Address)

        cmdExecute.Text = START_SERVER


        ' specifies the minimal size of the form (pixels width x height)
        Me.MinimumSize = New Size(750, 640)
    End Sub




    ''' <summary>
    ''' frmMain_FormClosing() launches when the form is closing. The close reason will
    ''' be checked, and the server will be stopped if necessary.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub frmMain_FormClosing(sender As Object, e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        ' First save our settings (right now we don't care about validity)
        saveSettingsToRegistry()

        ' Allow user to cancel - except when Windows is shutting down
        If e.CloseReason <> CloseReason.WindowsShutDown Then

            If myServer.isRunning() Then

                Dim ans As MsgBoxResult
                ans = MsgBox("The server is currently running. Are you sure you want to close the program? " & _
                             "This will shutdown the server and terminate all active connections.",
                              MsgBoxStyle.YesNoCancel Or MsgBoxStyle.DefaultButton3 Or MsgBoxStyle.Question,
                              "Quit server?")

                e.Cancel = (ans <> MsgBoxResult.Yes)
            End If
        End If

        If Not e.Cancel Then
            sysTray.Visible = False
        End If
    End Sub




    ''' <summary>
    ''' frmMain_Resize() is triggered when the form is resized or the minimize/maximize
    ''' windowstate is changed.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub frmMain_Resize() Handles Me.Resize
        If Me.WindowState = FormWindowState.Minimized Then
            ' Form is minimized now. Put to tray if tray icon is enabled
            If chkSysTray.Checked Then
                Me.Visible = False
                Me.Hide()
            End If
        Else
            ' Form is not minimized. Make sure the form is shown and the
            ' layout is recalculated
            If Not Me.Visible Then
                Me.Show()
            End If
            setLayout()
        End If
    End Sub




    ''' <summary>
    ''' frmMain_Shown() is triggered when the form is shown for the very first time.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub frmMain_Shown() Handles Me.Shown
        mIsLoaded = True

        ' Set initial width of log list. For some reason needed because 
        ' docking of loglist is not right when first launching the program.
        ' TODO: figure out why docking fails
        lstLogFile.Width = frame_K8055.Width - 10

        ' Connect to K8055 card
        tryToConnect()
        validateSettings()

        ' AutoRun server if required
        If mK8055.isConnected() And chkAutoRunServer.Checked Then
            cmdExecute_Click()
        End If
    End Sub




    ''' <summary>
    ''' prepareLists() will prepare all of the listviews.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub prepareLists()
        ' This will prepare our listview
        ' Set listview properties
        With lstLogFile
            .View = View.Details
            .GridLines = True
            .FullRowSelect = True
            .Columns.Add("Date", 125, HorizontalAlignment.Left)
            .Columns.Add("ID", 30, HorizontalAlignment.Left)
            .Columns.Add("IP", 100, HorizontalAlignment.Left)
            .Columns.Add("Event", 200, HorizontalAlignment.Left)
        End With


        With lstCurrentClients
            .View = View.Details
            .GridLines = True
            .FullRowSelect = True
            .Columns.Add("ID", 35, HorizontalAlignment.Left)
            .Columns.Add("IP", 120, HorizontalAlignment.Left)
        End With
    End Sub




    ''' <summary>
    ''' loadAddresses() will load all available K8055 addresses to the combobox. The
    ''' listindex of each item corresponds with the card number.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub loadAddresses()
        With cmbAddresses
            .Items.Add("0 (SK5=ON, SK6=ON)")
            .Items.Add("1 (SK5=OFF, SK6=ON)")
            .Items.Add("2 (SK5=ON, SK6=OFF)")
            .Items.Add("3 (SK5=OFF, SK6=OFF)")

            ' Select saved setting (if valid)
            If My.Settings.Address >= 0 And My.Settings.Address <= 3 Then
                .SelectedIndex = My.Settings.Address
            End If
        End With
    End Sub





    ''' <summary>
    ''' outputCheckboxChanged() is called when the state of one of the output checkboxes
    ''' is changed. 
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub outputCheckboxChanged(sender As System.Object, e As System.EventArgs) Handles OUT1.CheckedChanged,
                                                                                            OUT2.CheckedChanged,
                                                                                            OUT3.CheckedChanged,
                                                                                            OUT4.CheckedChanged,
                                                                                            OUT5.CheckedChanged,
                                                                                            OUT6.CheckedChanged,
                                                                                            OUT7.CheckedChanged,
                                                                                            OUT8.CheckedChanged

        If mK8055.isConnected() Then
            For i As Integer = 0 To mOutputCheckboxes.Count - 1
                ' find corresponding checkbox in array and set the output to desired state
                If mOutputCheckboxes(i).Equals(sender) Then
                    If mOutputCheckboxes(i).Checked Then
                        mK8055.setDigitalOutput(i + 1)
                    Else
                        mK8055.clearDigitalOutput(i + 1)
                    End If
                End If
            Next

        Else
            MsgBox("The K8055 is not connected right now",
                    MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation,
                    "No 8055 connected")
        End If 'myK8055.isConnected()
    End Sub




    ''' <summary>
    ''' enableFormInputs() will enable or disable all of the form's text inputs 
    ''' </summary>
    ''' <param name="state">A boolean with the desired state</param>
    ''' <remarks></remarks>
    Private Sub enableFormInputs(state As Boolean)
        For Each input As TextBox In mInputTextboxes
            input.Enabled = state
        Next

        cmbAddresses.Enabled = state
    End Sub




    ''' <summary>
    ''' checkboxChanged() is launched when one of the option checkboxes is changed.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub checkboxChanged() Handles chkReadOnly.CheckedChanged,
                                          chkSysTray.CheckedChanged,
                                          chkAutoRunServer.CheckedChanged,
                                          chkEnableTelnet.CheckedChanged
        serverInterpreter.readOnly = chkReadOnly.Checked
        sysTray.Visible = chkSysTray.Checked

        ' Pass read/write acccess permission to K8055 object
        mK8055.allowWriteAccess = Not chkReadOnly.Checked
        validateSettings()
    End Sub




    ''' <summary>
    ''' setLayout() will configure the layout according the actual size of the GUI
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub setLayout()
        Dim spacing As Integer = 10

        frame_K8055.Left = spacing
        frame_options.Left = spacing
        frame_server.Left = spacing
        frame_log.Left = frame_K8055.Left + frame_K8055.Width + spacing
        frame_log.Width = Me.ClientSize.Width - frame_K8055.Left - frame_log.Left
        frame_log.Height = Me.ClientSize.Height - frame_log.Top - 3 * spacing
    End Sub




    ''' <summary>
    ''' sysTray_Click() is is triggerd when the user clicks or double clicks the icon in the
    ''' system tray.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub sysTray_Click() Handles sysTray.Click,
                                        sysTray.MouseDoubleClick
        If Me.Visible = False Then
            Me.Show()
            Me.WindowState = FormWindowState.Normal
        End If
    End Sub




    ''' <summary>
    ''' updateCheckBoxes() is an aider function that updates the 8 output checkboxes to make
    ''' them match the actual K8055 output state.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub updateCheckBoxes()
        For i As Integer = 0 To 7
            mOutputCheckboxes(i).Checked = mK8055.getDigitalOutput(i + 1)
        Next
    End Sub





    ''' <summary>
    ''' callback_instruction() receives instructions from the client connections: it is called each time
    ''' the server receives a new line from a client.
    ''' </summary>
    ''' <param name="clientID">The clientID that triggered the instruction</param>
    ''' <param name="command">The command name</param>
    ''' <param name="parameter">The command parameter</param>
    ''' <returns>The response that must be sent back to the client</returns>
    ''' <remarks></remarks>
    Public Function callback_instruction(clientID As Integer, command As String, parameter As String) As String

        If Me.InvokeRequired Then
            Dim target As Func(Of Integer, String, String, String) = AddressOf callback_instruction

            Return DirectCast(
                Me.Invoke(target, clientID, command, parameter),
                String)
        Else
            ' TODO: commands are now being executed on GUI thread. Heavy traffic might slow down code.
            Dim res As String
            res = serverInterpreter.executeServerCommand(command, parameter)
            updateCheckBoxes()
            Return res
        End If

        ' Here a function is executed on the main thread
    End Function




    ''' <summary>
    ''' cmbAddresses_SelectedIndexChanged() is triggered when the user changes the K8055 hardware address.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub cmbAddresses_SelectedIndexChanged() Handles cmbAddresses.SelectedIndexChanged
        My.Settings.Address = cmbAddresses.SelectedIndex
        If mIsLoaded Then
            ' Only execute when form is already initialized
            tryToConnect()
            validateSettings()
        End If
    End Sub
#End Region






#Region "Communication and Decoding"

    ''' <summary>
    ''' getcheckers will return a String with 8 characters, containing only 1 or 0's. It represents the desired input state.
    ''' </summary>
    ''' <returns>A 8-character string which represents the desired output state.</returns>
    ''' <remarks></remarks>
    Private Function cChannelsToString() As String
        Dim response As String = String.Empty
        For Each check As CheckBox In mOutputCheckboxes
            If check.Checked Then
                response += "1"
            Else
                response += "0"
            End If
        Next
        Return response
    End Function





    ''' <summary>
    ''' GetIPv4Address() will output the IP address - in case the computer has multiple addresses,
    ''' it will select the first result.
    ''' </summary>
    ''' <returns>The first IP address the system could find</returns>
    ''' <remarks></remarks>
    Private Function GetIPv4Address() As String
        GetIPv4Address = String.Empty
        Dim strHostName As String = System.Net.Dns.GetHostName()
        Dim iphe As System.Net.IPHostEntry = System.Net.Dns.GetHostEntry(strHostName)

        For Each ipheal As System.Net.IPAddress In iphe.AddressList
            If ipheal.AddressFamily = System.Net.Sockets.AddressFamily.InterNetwork Then
                Return ipheal.ToString()
            End If
        Next

        Return String.Empty
    End Function




    ''' <summary>
    ''' setLblExternal() sets the text of the external label
    ''' </summary>
    ''' <param name="text">The text to be set</param>
    ''' <remarks></remarks>
    Private Sub setLblExternal(text As String)
        If lblExternalIP.InvokeRequired Then
            Dim invoker As Action(Of String) = AddressOf setLblExternal
            Me.Invoke(invoker, text)
        Else
            lblExternalIP.Enabled = True
            lblExternalIP.Text = String.Format("External IP: {0}", text)
        End If

    End Sub




    ''' <summary>
    ''' lblExternalIP_LinkClicked() handles the code when clicking the link to update the external IP
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub lblExternalIP_LinkClicked() Handles lblExternalIP.LinkClicked
        ' Define a background thread that connects to the Internet to calculate your external IP address
        Dim bgThread As New Thread(Sub()
                                       Try
                                           Dim MyWebClient As New System.Net.WebClient()
                                           setLblExternal(MyWebClient.DownloadString(REMOTE_IP_URL).ToString())
                                       Catch ex As Exception
                                           setLblExternal("Error: unable to retrieve IP. Click to retry.")
                                       End Try
                                   End Sub)

        ' Disables the label to avoid people click it too often. Then execute thread.
        lblExternalIP.Enabled = False
        bgThread.Start()
    End Sub
#End Region





#Region "Input Fields and Validation"

    ''' <summary>
    ''' textBoxChanged is an eventhandler which is raised when one of the textboxes is changed. Then the data will
    ''' automatic be saved into the Registry.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub textFieldChanged() Handles inpPassword.TextChanged,
                                            inpUser.TextChanged,
                                            inpPort.TextChanged

        validateSettings()
    End Sub




    ''' <summary>
    ''' validateSettings() validates all of the form's inputs and returns true if the 
    ''' settings are correct.
    ''' </summary>
    ''' <returns>A boolean to indicate the validity of the form's inputs</returns>
    ''' <remarks></remarks>
    Private Function validateSettings() As Boolean
        ' Set a boolean to indicate all fields are valid. Value is cleared in case of an error.
        Dim allFieldsValid As Boolean = True
        ' Reset ErrorProvider
        EP.Clear()

        Dim port As Integer
        If Integer.TryParse(inpPort.Text, port) Then
            If port < 1 Or port > 65535 Then
                allFieldsValid = False
                EP.SetError(inpPort, "Set a port number between 1 and 65535")
            End If
        Else
            allFieldsValid = False
            EP.SetError(inpPort, "Set a port number between 1 and 65535")
        End If

        If inpUser.Text.Length() < 4 Then
            allFieldsValid = False
            EP.SetError(inpUser, "Please fill in a valid username that contains at least 4 characters")
        End If

        If inpPassword.Text.Trim().Length() < 4 Then
            allFieldsValid = False
            EP.SetError(inpPassword, "Please fill in a valid username that contains at least 4 characters")
        End If

        If Not mK8055.isConnected() Then
            EP.SetError(cmbAddresses, "K8055 is not connected. Set the right address or check the USB connection")
        End If

        Return allFieldsValid
    End Function
#End Region





#Region "Read and Write settings"
    ''' <summary>
    ''' saveSettingsToRegistry will write all settings to the Registry.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub saveSettingsToRegistry()
        With My.Settings
            .Password = inpPassword.Text
            .Username = inpUser.Text
            .Port = inpPort.Text
            .AutoStart = chkAutoRunServer.Checked
            .isReadonly = chkReadOnly.Checked
            .minimizeToTray = chkSysTray.Checked
            .enableTelnet = chkEnableTelnet.Checked

            ' Save setting immediately 
            .Save()
        End With
    End Sub




    ''' <summary>
    ''' readFromRegistry reads all settings from the Registry.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub readFromRegistry()
        With My.Settings
            inpPassword.Text = .Password
            inpUser.Text = .Username
            inpPort.Text = .Port
            chkAutoRunServer.Checked = .AutoStart
            chkReadOnly.Checked = .isReadonly
            chkSysTray.Checked = .minimizeToTray
            chkEnableTelnet.Checked = .enableTelnet
        End With
    End Sub
#End Region





#Region "K8055 access routines"
    ''' <summary>
    ''' setCheckBoxes() will set all K8055's checkboxes to the desired state.
    ''' The state must be passed as a string.
    ''' </summary>
    ''' <param name="outputs">A string with the desired outputs</param>
    ''' <remarks></remarks>
    Private Sub setCheckBoxes(outputs As String)
        If Me.mOutputCheckboxes(0).InvokeRequired Then
            Dim target As Action(Of String) = AddressOf setCheckBoxes
            Me.Invoke(target, outputs) 'New Object() {outputs})
        Else
            For i As Integer = 0 To 7
                mOutputCheckboxes(i).Checked = (outputs.Chars(i) = "1")
            Next
        End If
    End Sub




    ''' <summary>
    ''' tryToConnect() tries to connect to the K8055 card.
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function tryToConnect() As Boolean

        Dim address As Integer = My.Settings.Address
        Dim repeat As Boolean = True

        While (repeat)
            If (address > -1 And address < 4) Then
                mK8055.disconnect()
                Dim success As Boolean = mK8055.connect(address)
                repeat = Not success

                If repeat Then
                    Dim res As MsgBoxResult

                    res = MsgBox(String.Format("The K8055 could not be found. Make sure the card is connected " & _
                                             "to your computer and the jumpers are set to {0}. Retry?", cmbAddresses.Text),
                                         MsgBoxStyle.YesNoCancel Or MsgBoxStyle.DefaultButton1 Or MsgBoxStyle.Exclamation,
                                         "Unable to connect K8055")

                    repeat = (res = MsgBoxResult.Yes)
                End If
            End If
        End While

        ' set text in status bar
        If mK8055.isConnected Then
            status.Text = "K8055 detected ..."
        Else
            status.Text = "No K8055 detected ..."
        End If

        ' return connection state to caller
        Return mK8055.isConnected()
    End Function

#End Region





#Region "Server Functions & Event Handlers"
    ''' <summary>
    ''' cmdExecute_Click() handles the click on the Execute/Stop button.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub cmdExecute_Click() Handles cmdExecute.Click
        ' Temporary disable enable-button (will be reactivated during events)
        cmdExecute.Enabled = False

        If myServer.isRunning() Then
            ' Server is already running > request stop
            myServer.stop()

        Else
            If Not mK8055.isConnected() Then
                tryToConnect()
            Else

            End If
            If validateSettings() AndAlso mK8055.isConnected() Then
                With myServer
                    .supportsTelnet = chkEnableTelnet.Checked
                    .server_user = inpUser.Text
                    .server_pass = inpPassword.Text
                    .server_port = Integer.Parse(inpPort.Text)  ' OK because validated

                    enableFormInputs(False)
                    .start()
                    cmdExecute.Text = STOP_SERVER
                End With

            Else
                cmdExecute.Enabled = True
                MsgBox("The server cannot start. Make sure all fields are filled " & _
                        "in correctly and the K8055 is connected!",
                        MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation)
            End If
        End If
    End Sub




    ''' <summary>
    ''' myServer_client_signal_received() is called when the server thread wants to signal an event.
    ''' </summary>
    ''' <param name="clientID">The ID of the client connection that triggered the event</param>
    ''' <param name="eventType">The type of signal that occured</param>
    ''' <remarks></remarks>
    Private Sub myServer_client_signal_received(clientID As Integer, eventType As ClientConnection.Signals) Handles myServer.client_signal_received

        Select Case eventType
            Case ClientConnection.Signals.CONNECTION_TERMINATED
                removeConnectedClientFromList(clientID)

            Case ClientConnection.Signals.USER_LOGGED_IN
                ' not implemented yet
        End Select
    End Sub




    ''' <summary>
    ''' serverStarted() is an event that is called when the server starts.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub serverStarted() Handles myServer.server_started
        If Me.InvokeRequired Then
            Dim target As Action = AddressOf serverStarted
            Me.Invoke(target)
        Else
            chkEnableTelnet.Enabled = False
            cmdExecute.Text = STOP_SERVER
            cmdExecute.Enabled = True
        End If
    End Sub




    ''' <summary>
    ''' serverStopped() is an event that is called when the server stops
    ''' </summary>
    ''' <param name="reason">A string with the reason why the server stopped</param>
    ''' <remarks></remarks>
    Private Sub serverStopped(reason As String) Handles myServer.server_stopped
        If Me.InvokeRequired Then
            Dim target As Action(Of String) = AddressOf serverStopped
            Me.Invoke(target, reason)
        Else
            chkEnableTelnet.Enabled = True
            cmdExecute.Text = START_SERVER
            enableFormInputs(True)
            cmdExecute.Enabled = True

            MsgBox(String.Format("The server has been stopped: {0}", reason),
                   MsgBoxStyle.OkOnly Or MsgBoxStyle.Information,
                   "Server stopped")
        End If
    End Sub




    ''' <summary>
    ''' insertLogLine() will append a new logline to the list
    ''' </summary>
    ''' <param name="clientID">The ID of the client that triggered the log</param>
    ''' <param name="IP">The IP of the corresponding client</param>
    ''' <param name="message">The log message</param>
    ''' <remarks></remarks>
    Private Sub insertLogLine(clientID As String, IP As String, message As String) Handles myServer.log_received
        If lstLogFile.InvokeRequired Then
            Dim target As Action(Of String, String, String) = AddressOf insertLogLine
            lstLogFile.Invoke(target, clientID, IP, message)
        Else
            Dim itmX As New ListViewItem
            itmX.Text = DateTime.Now.ToString()
            itmX.SubItems.Add(clientID)
            itmX.SubItems.Add(IP)
            itmX.SubItems.Add(message)
            lstLogFile.Items.Insert(0, itmX)

            ' Append Logfile as well
            addToLogFile(String.Format("{0};{1};{2};{3}",
                                       DateTime.Now.ToString(),
                                       clientID,
                                       IP,
                                       message))

            ' Limit the number of items in the list. Max lines is defined in constant MAX_LISTITEMS
            If lstLogFile.Items.Count > MAX_LISTITEMS Then
                lstLogFile.Items.RemoveAt(lstLogFile.Items.Count - 1)
            End If
        End If
    End Sub




    ''' <summary>
    ''' addConnectedClientToList() is called when a client connects to the server
    ''' </summary>
    ''' <param name="clientID">The ID of the client</param>
    ''' <param name="IP">The IP address of the client</param>
    ''' <remarks></remarks>
    Private Sub addConnectedClientToList(clientID As Integer, IP As String) Handles myServer.client_connected
        cmdExecute.Enabled = True

        If lstCurrentClients.InvokeRequired Then
            Dim target As Action(Of Integer, String) = AddressOf addConnectedClientToList
            Me.Invoke(target, clientID, IP)
        Else
            ' Add newly created client to the list and select last item
            Dim newLine As New ListViewItem
            newLine.Text = clientID.ToString()
            newLine.SubItems.Add(IP)
            lstCurrentClients.Items.Add(newLine)
            lstCurrentClients.Items(lstCurrentClients.Items.Count - 1).Selected = True
        End If
    End Sub




    ''' <summary>
    ''' removeConnectedClientFromList() is called when a client disconnects from the server
    ''' </summary>
    ''' <param name="clientID"></param>
    ''' <remarks></remarks>
    Private Sub removeConnectedClientFromList(clientID As Integer) Handles myServer.client_disconnected

        If lstCurrentClients.InvokeRequired Then
            Dim target As Action(Of Integer) = AddressOf removeConnectedClientFromList
            Me.Invoke(target, clientID)
        Else
            cmdExecute.Enabled = True

            For i As Integer = lstCurrentClients.Items.Count - 1 To 0 Step -1
                If lstCurrentClients.Items(i).Text = clientID.ToString() Then
                    lstCurrentClients.Items.RemoveAt(i)
                End If
            Next
        End If
    End Sub
#End Region





#Region "Log Management"
    ''' <summary>
    ''' mnuClearList_Click() clears all items from the logger list
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub mnuClearList_Click() Handles mnuClearList.Click
        Dim res As MsgBoxResult
        res = MsgBox("Are you sure you want to erase the complete list?",
                      MsgBoxStyle.YesNoCancel Or MsgBoxStyle.Question Or MsgBoxStyle.DefaultButton3,
                      "Clear List?")

        If res = MsgBoxResult.Yes Then
            lstLogFile.Items.Clear()
        End If
    End Sub



    ''' <summary>
    ''' addToLogFile() adds a new message to the logfile
    ''' </summary>
    ''' <param name="text">The message to be added</param>
    ''' <remarks></remarks>
    Private Sub addToLogFile(text As String)
        Try
            Dim tmpFile As String = getOrCreateLogFile()

            Using sw As New StreamWriter(File.Open(tmpFile, FileMode.Append))
                sw.WriteLine(text)
                sw.Close()
            End Using
        Catch ex As Exception
            ' For now we don't care about log errors
        End Try
    End Sub



    ''' <summary>
    ''' ShellExecute() executes a file with its default program
    ''' </summary>
    ''' <param name="File">the file to be opened</param>
    ''' <remarks></remarks>
    Private Sub ShellExecute(File As String)
        Dim myProcess As New Process

        With myProcess
            .StartInfo.FileName = File
            .StartInfo.UseShellExecute = True
            .StartInfo.RedirectStandardOutput = False
            .Start()
            .Dispose()
        End With
    End Sub




    ''' <summary>
    ''' openLogFile() opens the logfile with its default program
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub openLogFile()
        Try
            Dim logFile As String = getOrCreateLogFile()
            ShellExecute(logFile)
        Catch ex As Exception
            MsgBox(String.Format("Unable to open file: {0}",
                                 ex.Message),
                   MsgBoxStyle.OkOnly Or MsgBoxStyle.Exclamation,
                   "Error while opening logfile")
        End Try
    End Sub




    ''' <summary>
    ''' getOrCreateLogFile() gets the current logfile, or creates a new one.
    ''' </summary>
    ''' <returns>The logfile currently assigned to the program</returns>
    ''' <remarks></remarks>
    Private Function getOrCreateLogFile() As String
        If Not File.Exists(My.Settings.logFile) Then
            ' File does not exist already. Create temporary filename
            Dim newFile As String = String.Format("{0}.csv", Path.GetTempFileName())
            My.Settings.logFile = newFile
        End If

        Return My.Settings.logFile
    End Function





    ''' <summary>
    ''' mnuOpenLogFile_Click() handles clicks on the openlogfile menu.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub mnuOpenLogFile_Click() Handles mnuOpenLogFile.Click
        openLogFile()
    End Sub
#End Region

End Class