﻿Option Explicit On
Option Strict On

Imports Microsoft.VisualBasic.FileIO

''' <summary>
''' A Serverfunction object contains all information about a certain function: the
''' name and the expected parameters.
''' 
''' The object also contains a function parameterMatches that will check the
''' parameter string (entered from the server) against the actual parameters.
''' </summary>
''' <remarks>Copyright (c) 2015   Vanderhaegen Bart</remarks>
Public Class ServerFunction

    ' Class variables
    Private functionName As String
    Private parameterTypes As Type()




    ''' <summary>
    ''' Class constructor
    ''' </summary>
    ''' <param name="Name">The name of the object</param>
    ''' <param name="parameterTypes">An array with the parameter types for each
    ''' of the parameters.</param>
    ''' <remarks></remarks>
    Public Sub New(Name As String, parameterTypes As Type())

        Me.functionName = Name.Trim().ToUpper()
        Me.parameterTypes = parameterTypes
    End Sub




    ''' <summary>
    ''' getName() returns the name of the serverfunction
    ''' </summary>
    ''' <returns>The name</returns>
    ''' <remarks></remarks>
    Public Function getName() As String
        Return Me.functionName
    End Function




    ''' <summary>
    ''' getParameterTypes() returns an array with all parameter types.
    ''' </summary>
    ''' <returns>An array of type objects.</returns>
    ''' <remarks></remarks>
    Public Function getParameterTypes() As Type()
        Return Me.parameterTypes
    End Function




    ''' <summary>
    ''' getParameterType() returns the parameter type at the specified
    ''' index.
    ''' </summary>
    ''' <param name="index">The zero-based index</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function getParameterType(index As Integer) As Type
        If index < 0 Or index > UBound(parameterTypes) Then
            Return Nothing
        Else
            Return parameterTypes(index)
        End If
    End Function




    ''' <summary>
    ''' getFunctionCount() gets the number of defined parameters.
    ''' </summary>
    ''' <returns>An integer containing the number of parameters.</returns>
    ''' <remarks></remarks>
    Public Function getParameterCount() As Integer
        If parameterTypes Is Nothing Then
            Return 0
        Else
            Return parameterTypes.Count
        End If
    End Function




    ''' <summary>
    ''' requiresParameter() returns a boolean that indicates if the function
    ''' has any parameters.
    ''' </summary>
    ''' <returns>True if the parametercount is zero.</returns>
    ''' <remarks></remarks>
    Public Function requiresParameter() As Boolean
        Return getParameterCount() > 0 'parameterTypes IsNot Nothing AndAlso parameterTypes.Count > 0
    End Function




    ''' <summary>
    ''' 
    ''' </summary>
    ''' <param name="fullParameterLine"></param>
    ''' <param name="dParameter"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function parameterMatches(fullParameterLine As String,
                                     ByRef dParameter() As CommandParameter) As Boolean

        If Not requiresParameter() Then
            Return (fullParameterLine = String.Empty)

        Else
            ReDim dParameter(getParameterCount() - 1)
            Dim ok As Boolean = True
            Dim ourVariables() As String = splitLine(fullParameterLine)
            ' If ourVariables Is Nothing Then ourVariables = {} 'todo : improve

            ' Check if segment count corresponds with expected count
            If ourVariables.Count = getParameterCount() Then
                For i As Integer = 0 To getParameterCount() - 1
                    dParameter(i) = DirectCast(
                                        Activator.CreateInstance(getParameterType(i), ourVariables(i)), 
                                        CommandParameter)

                    ok = ok And dParameter(i).isValid()
                Next
                Return ok
            Else
                Return False
            End If
        End If
    End Function




    ''' <summary>
    ''' The function splitLine() will split a text string in segments.
    ''' </summary>
    ''' <param name="line"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function splitLine(line As String) As String()
        Dim fields() As String

        Using rdr As New IO.StringReader(line)
            Using parser As New TextFieldParser(rdr)
                parser.TextFieldType = FieldType.Delimited
                parser.Delimiters = New String() {" "}
                parser.HasFieldsEnclosedInQuotes = True
                fields = parser.ReadFields()
            End Using
        End Using

        ' Make sure function never outputs NULL
        If fields Is Nothing Then
            fields = {}
        End If

        Return fields
    End Function

End Class